# Re-export from discord_slash
from discord_slash.context import SlashContext
from discord_slash.model import SlashCommandOptionType
from discord_slash.utils import manage_commands

# Export own functions and classes
from .client import SlashCommand
from .converters import convert
from .decorators import slash, slash_cog

__version__ = "1.1.1"
